@@ -327,6 +327,75 @@ class ActuationModel(Model):
 
         return uim, pum, tst
 
+    def actuation_stage_residual(self, frequencies, stage='TST'):
+        """
+        This is the actuation residual for a given stage on a given arm (this
+        object), meaning it contains everything in the actuation except for
+        the output matrix, the digital distribution filters, the actuator
+        gain, and suspension dynamics.
+
+        See T1900169
+
+        Parameters
+        ----------
+        frequencies : `float`, array-like
+            array of frequencies to compute the response
+        stage : `str`, optional
+            UIM, PUM or TST (default = 'TST')
+
+        Returns
+        -------
+        A_mn_res : `complex128`, array-like
+            transfer function response of the residual actuation function
+        """
+
+        # SUS to IOP and IOP to analog digital time delays (see G1601472)
+        sus_to_iop_delay_response = signal.dfreqresp(
+            digital_delay_filter(1, 16384), 2.0*np.pi*frequencies/16384)[1]
+        iop_to_analog_delay_response = signal.dfreqresp(
+            digital_delay_filter(4, 65536), 2.0*np.pi*frequencies/65536)[1]
+
+        # digital anti-imaging filter
+        digital_ai_filter_response = \
+            self.digital_aa_or_ai_filter_response(frequencies)
+
+        # analog anti-imaging filter response
+        analog_ai_filter_response = \
+            self.analog_aa_or_ai_filter_response(frequencies)
+
+        # Combine analog driver electronics for each quadrant with the
+        # digital compensation for each quadrant (if specified), and
+        # then take the mean of the four quadrants for each stage. Most
+        # often, this only contains the high frequency response that we
+        # cannot compensate.
+        # Order: [UIM, PUM, TST]
+        stages_tfs = self.combine_sus_quadrants(frequencies)
+        stage_index = ['uim', 'pum', 'tst'].index(stage.lower())
+        stage_tf = stages_tfs[stage_index]
+
+        # Unknown overall time delay
+        unknown_actuation_delay = np.exp(-2.0*np.pi*1j *
+                                         self.unknown_actuation_delay *
+                                         frequencies)
+
+        # Independent time delay
+        if (hasattr(self, f'{stage.lower()}_delay') and
+                getattr(self, f'{stage.lower()}_delay') != ''):
+            delay = getattr(self, f'{stage.lower()}_delay')
+            stage_delay = np.exp(-2.0*np.pi*1j*delay*frequencies)
+        else:
+            stage_delay = 1
+
+        A_mn_res = (sus_to_iop_delay_response *
+                    iop_to_analog_delay_response *
+                    digital_ai_filter_response *
+                    analog_ai_filter_response *
+                    stage_tf *
+                    unknown_actuation_delay *
+                    stage_delay)
+
+        return A_mn_res
+
     @staticmethod
     def matlab_force2length_response(suspension_file, frequencies):
         """
@@ -366,7 +435,7 @@ class ActuationModel(Model):
 
     def digital_out_to_displacement(self, frequencies):
         """
-        This computes the transfer function from the output of the drive align
+        This computes the transfer function from the output of the DRIVEALIGN
         filter bank of each stage to displacement of the test mass
 
         Parameters
@@ -384,25 +453,6 @@ class ActuationModel(Model):
             transfer function response of the TST stage
         """
 
-        # SUS to IOP and IOP to analog digital time delays (see G1601472)
-        sus_to_iop_delay_response = signal.dfreqresp(
-            digital_delay_filter(1, 16384), 2.0*np.pi*frequencies/16384)[1]
-        iop_to_analog_delay_response = signal.dfreqresp(
-            digital_delay_filter(4, 65536), 2.0*np.pi*frequencies/65536)[1]
-
-        # digital anti-imaging filter
-        digital_ai_filter_response = \
-            self.digital_aa_or_ai_filter_response(frequencies)
-
-        # analog anti-imaging filter response
-        analog_ai_filter_response = \
-            self.analog_aa_or_ai_filter_response(frequencies)
-
-        # Combine analog driver electronics for each quadrant with the
-        # digital compensation for each quadrant, and then take the mean
-        # of the four quadrants for each stage
-        uim_tf, pum_tf, tst_tf = self.combine_sus_quadrants(frequencies)
-
         # Analog force to length response for each stage from Matlab
         sus_file = self.dpath(self.suspension_file)
         [uim_f2l_response,
@@ -410,62 +460,33 @@ class ActuationModel(Model):
          tst_f2l_response] = self.matlab_force2length_response(sus_file,
                                                                frequencies)
 
-        # Unknown overall time delay
-        unknown_actuation_delay = np.exp(-2.0*np.pi*1j *
-                                         self.unknown_actuation_delay *
-                                         frequencies)
-
-        # Independent time delays
-        if hasattr(self, 'uim_delay') and self.uim_delay != '':
-            uim_delay_tf = np.exp(-2.0*np.pi*1j*self.uim_delay*frequencies)
-        else:
-            uim_delay_tf = 1
-        if hasattr(self, 'pum_delay') and self.pum_delay != '':
-            pum_delay_tf = np.exp(-2.0*np.pi*1j*self.pum_delay*frequencies)
-        else:
-            pum_delay_tf = 1
-        if hasattr(self, 'tst_delay') and self.tst_delay != '':
-            tst_delay_tf = np.exp(-2.0*np.pi*1j*self.tst_delay*frequencies)
-        else:
-            tst_delay_tf = 1
+        # Actuation stage residuals
+        A_u_res = self.actuation_stage_residual(frequencies, stage='UIM')
+        A_p_res = self.actuation_stage_residual(frequencies, stage='PUM')
+        A_t_res = self.actuation_stage_residual(frequencies, stage='TST')
 
         # Everything together now!
-        tst_response = (tst_tf *
-                        sus_to_iop_delay_response *
-                        iop_to_analog_delay_response *
-                        digital_ai_filter_response *
-                        analog_ai_filter_response *
+        tst_response = (A_t_res *
                         self.tst_dc_gain_Npct() *
-                        tst_f2l_response *
-                        unknown_actuation_delay *
-                        tst_delay_tf)
-        pum_response = (pum_tf *
-                        sus_to_iop_delay_response *
-                        iop_to_analog_delay_response *
-                        digital_ai_filter_response *
-                        analog_ai_filter_response *
+                        tst_f2l_response)
+        pum_response = (A_u_res *
                         self.pum_dc_gain_Npct() *
-                        pum_f2l_response *
-                        unknown_actuation_delay *
-                        pum_delay_tf)
-        uim_response = (uim_tf *
-                        sus_to_iop_delay_response *
-                        iop_to_analog_delay_response *
-                        digital_ai_filter_response *
-                        analog_ai_filter_response *
+                        pum_f2l_response)
+        uim_response = (A_p_res *
                         self.uim_dc_gain_Npct() *
-                        uim_f2l_response *
-                        unknown_actuation_delay *
-                        uim_delay_tf)
+                        uim_f2l_response)
 
         return uim_response, pum_response, tst_response
 
-    def drivealign_to_displacement(self, frequencies):
+    def drivealign_to_longitudinal_displacement(self, frequencies):
         """
-        This computes the transfer function from the input of the drive align
-        filter bank of each stage to displacement of the test mass.
-
-        This is particularly useful for computing EPICS records
+        This computes the transfer function from the input of the DRIVEALIGN
+        filter bank of each stage to displacement of the test mass
+        where "longitudinal" is defined as in the traditional SUS
+        Euler Basis: positive longitudinal displacement is along a
+        vector perpendicular to the plane of the HR surface
+        (positive ETMX displacement is in the -X global IFO direction,
+         positive ITMY displacement is in the +Y global IFO direction).
 
         Parameters
         ----------
@@ -513,16 +534,58 @@ class ActuationModel(Model):
 
         return uim_response, pum_response, tst_response
 
+    def drivealign_to_darm_displacement(self, frequencies):
+        """
+        This computes the transfer function from the input of the DRIVEALIGN
+        filter bank of each action stage to DARM by accounting for how
+        positive longitudinal drive for each actuation stage pushes the given
+        test mass to creates DARM.
+        DARM displacement is defined by
+        + DARM = + Delta L
+               = (Delta L_X - Delta L_Y)
+               = - (Delta L_EX + Delta L_IX)
+                    + (Delta L_EY + Delta L_IY).
+        This is particularly useful for computing EPICS records
+
+        Parameters
+        ----------
+        frequencies : `float`, array-like
+            array of frequencies to compute the response
+
+        Returns
+        -------
+        uim_response : `complex128`, array-like
+            transfer function response of the UIM stage
+        pum_response : `complex128`, array-like
+            transfer function response of the PUM stage
+        tst_response : `complex128`, array-like
+            transfer function response of the TST stage
+        """
+
+        longitudinal_displacement = (
+            self.drivealign_to_longitudinal_displacement(frequencies))
+
+        # Multiply each stage by the DARM feedback sign
+        for idx, tf in enumerate(longitudinal_displacement):
+            tf *= self.darm_feedback_sign
+        darm_displacement = longitudinal_displacement  # rename after DARM sign
+
+        # Return the DARM displacement tuple
+        return darm_displacement
+
     def known_actuation_terms_for_measurement(self, frequencies):
         """
-        This computes the transfer function from the input of the drive align
-        filter bank of each stage to displacement of the test mass, everything
-        but N/(drive unit); either amps or volts**2. This also includes the
-        DARM feedback sign of the arm the measurement is taken on since we
-        read out the measurment at DARM
+        This method computes all known terms which are divided out of the
+        DARM_ERR / SUS_EXC full IFO actuation measurement for each stage
+        such that the remaining dimensional actuation strength, H_Ai, can
+        be fit by MCMC.
 
-        This is used for dividing out everything from measurements to establish
-        the actuator gain in units of N/(drive unit)
+        This computes the transfer function from the input of the DRIVEALIGN
+        filter bank (same as the SUS_EXC point) of each stage to DARM
+        displacement from any test mass, including everything except the
+        N/(drive unit); either amps or volts**2, for which the MCMC will fit.
+        The DARM feedback sign of the arm the measurement is taken on since
+        we read out the measurement with DARM_ERR.
 
         Parameters
         ----------
@@ -541,28 +604,29 @@ class ActuationModel(Model):
 
         [uim_response,
          pum_response,
-         tst_response] = self.drivealign_to_displacement(frequencies)
+         tst_response] = self.drivealign_to_darm_displacement(frequencies)
 
-        uim_response *= self.darm_feedback_sign * \
-            (self.uim_dc_gain_Apct()/self.uim_dc_gain_Npct())
-        pum_response *= self.darm_feedback_sign * \
-            (self.pum_dc_gain_Apct()/self.pum_dc_gain_Npct())
-        tst_response *= self.darm_feedback_sign * \
-            (self.tst_dc_gain_V2pct()/self.tst_dc_gain_Npct())
+        uim_response *= (self.uim_dc_gain_Apct() / self.uim_dc_gain_Npct())
+        pum_response *= (self.pum_dc_gain_Apct() / self.pum_dc_gain_Npct())
+        tst_response *= (self.tst_dc_gain_V2pct() / self.tst_dc_gain_Npct())
 
         return uim_response, pum_response, tst_response
 
     def compute_actuation_single_stage(self, frequencies, stage='TST'):
         """
         Compute the actuation function transfer function for a single stage
-        (see G1501518). This transfer function is from DARM_CTRL to meters
-        sensed by the IFO. Note that the sign of the DARM_ERR signal is
-        dependent upon which arm is under control.
+        (see G1501518). This transfer function is from the DARM control
+        input at the SUS user model, or the input to the ISCINF bank, to
+        DARM displacement.
+
+        This does not include the OMC to SUS model jump delay
 
         Parameters
         ----------
         frequencies : `float`, array-like
             array of frequencies to compute the response
+        stage : `str`, optional
+            UIM, PUM or TST (default = 'TST')
 
         Returns
         -------
@@ -572,40 +636,28 @@ class ActuationModel(Model):
 
         # Digital output to TST displacement
         # Note that no feedback or output matrix values are applied here
+        # Note also that digital_out_to_displacement does not have the
+        # OMC to SUS delay
         [uim_response,
          pum_response,
          tst_response] = self.digital_out_to_displacement(frequencies)
-        if stage == 'UIM' or stage == 'L1':
-            sus_response = uim_response
-        elif stage == 'PUM' or stage == 'L2':
-            sus_response = pum_response
-        elif stage == 'TST' or stage == 'L3':
-            sus_response = tst_response
-        else:
-            raise ValueError('stage must be UIM, PUM, TST, L1, L2, or L3')
 
         # All SUS digital filters
         [uim_digital_filter_response,
          pum_digital_filter_response,
          tst_digital_filter_response] = self.sus_digital_filters_response(
              frequencies)
-        if stage == 'UIM' or stage == 'L1':
-            digital_filter_response = uim_digital_filter_response
-        elif stage == 'PUM' or stage == 'L2':
-            digital_filter_response = pum_digital_filter_response
+
+        # Choose the right stage
+        if stage in ['UIM', 'L1']:
+            sus_response = uim_response * uim_digital_filter_response
+        elif stage in ['PUM', 'L2']:
+            sus_response = pum_response * pum_digital_filter_response
+        elif stage in ['TST', 'L3']:
+            sus_response = tst_response * tst_digital_filter_response
         else:
-            digital_filter_response = tst_digital_filter_response
+            raise ValueError('stage must be UIM, PUM, TST, L1, L2, or L3')
 
-        # OMC to SUS digital time delay (see G1601472)
-        # Note that SUS to IOP, IOP to analog, and unknown delays already
-        # applied
-        omc_to_sus_delay_response = signal.dfreqresp(
-            digital_delay_filter(1, 16384), 2.0*np.pi*frequencies/16384)[1]
-
-        sus_response *= digital_filter_response*omc_to_sus_delay_response
-
-        # The response on DARM will depend on which arm is being actuated
-        # by the DARM feedback
         sus_response *= self.darm_feedback_sign
 
         # Everything *above* here is modelling the physical actuator that we
@@ -720,4 +772,15 @@ class DARMActuationModel(Model):
 
                     darm_actuation += output_matrix[arm, stage] * single_stage
 
+        # OMC to SUS model jump delay, sending DARM_CTRL from the omc
+        # front-end model via IPC to any QUAD SUS model, identical for any
+        # QUAD, either at the corner or end.
+        # Note that SUS to IOP, IOP to analog, and unknown delays have already
+        # been applied in compute_actuation_single_stage.
+        # For measurement confirmation of computational time delays see
+        # G1601472.
+        omc_to_sus_delay_response = signal.dfreqresp(
+            digital_delay_filter(1, 16384), 2.0*np.pi*frequencies/16384)[1]
+        darm_actuation *= omc_to_sus_delay_response
+
         return darm_actuation
