@@ -17,12 +17,13 @@
 import numpy as np
 from scipy import signal
 from gwpy.time import tconvert, from_gps
+import time
 
 from .model import Model
 from .sensing import SensingModel
 from .actuation import DARMActuationModel
 from .pcal import PcalModel
-from .utils import compute_digital_filter_response
+from .utils import compute_digital_filter_response, digital_delay_filter
 from .digital import daqdownsamplingfilters
 from .plot import critique
 
@@ -138,7 +139,8 @@ class DARMModel(Model):
         return (1.0/C_response + D_response * A_response)
 
     def compute_epics_records(self, f_pcal1, f_uim, f_pum, f_tst, f_pcal2,
-                              f_pcal3, f_pcal4, arm, save_to_file=None):
+                              f_pcal3, f_pcal4, arm, save_to_file=None,
+                              endstation=False):
         """
         This is a LIGO specific function for generating front end calibration
         and GDS EPICS records. This function is very specific for O3, but we
@@ -167,8 +169,13 @@ class DARMModel(Model):
         arm : `str`
             'x' or 'y' to indicate which arm is used for the SUS actuators
         save_to_file : `str`, optional
-            Filename (ASCII) to save the values for EPICS from this reference model
-
+            Filename (ASCII) to save the values for EPICS from this reference
+            model
+        endstation : `bool`, optional
+            When false (default), the correction is computed for CAL-CS
+            PCAL channel, which includes 1 16k clock cycle delay that we must
+            compensate (undo). Otherwise, when true, the correction is computed
+            at the end station, which does not include 1 16k clock cycle delay.
 
         Returns
         -------
@@ -188,15 +195,15 @@ class DARMModel(Model):
 
         # Pcal corrections
         # See T1700106
-        pcal_correction = self.pcal.compute_pcal_correction(freq)
-        PCAL_LINE1_CORRECTION = pcal_correction[0] * \
-            np.exp(-2*np.pi*1j*freq[0]/16384)   # EP0
-        PCAL_LINE2_CORRECTION = pcal_correction[4] * \
-            np.exp(-2*np.pi*1j*freq[4]/16384)   # EP9
-        PCAL_LINE3_CORRECTION = pcal_correction[5] * \
-            np.exp(-2*np.pi*1j*freq[5]/16384)   # EP24
-        PCAL_LINE4_CORRECTION = pcal_correction[6] * \
-            np.exp(-2*np.pi*1j*freq[6]/16384)   # EP15
+        pcal_correction = self.pcal.compute_pcal_correction(freq, endstation)
+        PCAL_LINE1_CORRECTION = (
+            pcal_correction[0] * np.exp(-2*np.pi*1j*freq[0]/16384))   # EP0
+        PCAL_LINE2_CORRECTION = (
+            pcal_correction[4] * np.exp(-2*np.pi*1j*freq[4]/16384))   # EP9
+        PCAL_LINE3_CORRECTION = (
+            pcal_correction[5] * np.exp(-2*np.pi*1j*freq[5]/16384))   # EP24
+        PCAL_LINE4_CORRECTION = (
+            pcal_correction[6] * np.exp(-2*np.pi*1j*freq[6]/16384))   # EP15
 
         # The actuator lines are injected at the input to the DRIVEALIGN bank
         # Also get the complete actuation model
@@ -207,52 +214,58 @@ class DARMModel(Model):
         else:
             raise ValueError('Must provide value of x or y to argument arm')
 
-        uim_actuation_epics, pum_actuation_epics, tst_actuation_epics = \
-            this_arm.drivealign_to_displacement(freq)
-        uim_actuation_epics *= this_arm.darm_feedback_sign
-        pum_actuation_epics *= this_arm.darm_feedback_sign
-        tst_actuation_epics *= this_arm.darm_feedback_sign
+        [uim_actuation_epics,
+         pum_actuation_epics,
+         tst_actuation_epics] = this_arm.drivealign_to_darm_displacement(freq)
         uim = this_arm.compute_actuation_single_stage(freq, stage='UIM')
         pum = this_arm.compute_actuation_single_stage(freq, stage='PUM')
         tst = this_arm.compute_actuation_single_stage(freq, stage='TST')
 
+        # single stage actuation computation doesn't include the OMC to SUS
+        # delay so we will add that in here
+        omc_to_sus_response = signal.dfreqresp(
+            digital_delay_filter(1, 16384), 2.0*np.pi*freq/16384)[1]
+        uim *= omc_to_sus_response
+        pum *= omc_to_sus_response
+        tst *= omc_to_sus_response
+
         # Actuator EPICS
         # Need to divide out a DAQ downsampling filter from the reference A
         # values because channels sampled at 512 Hz have an AA filter applied
         # See T1700106
-        SUS_LINE1_REF_INVA_UIM_RESPRATIO = \
-            (1.0/(uim_actuation_epics[1]/daqdownsampling_response[1])) * \
-            (1.0/R[0]) * R[1] * np.exp(2*np.pi*1j*freq[1]/16384)  # EP3
-        SUS_LINE2_REF_INVA_PUM_RESPRATIO = \
-            (1.0/(pum_actuation_epics[2]/daqdownsampling_response[2])) * \
-            (1.0/R[0]) * R[2] * np.exp(2*np.pi*1j*freq[2]/16384)  # EP2
-        SUS_LINE3_REF_INVA_TST_RESPRATIO = \
-            (1.0/(tst_actuation_epics[3]/daqdownsampling_response[3])) * \
-            (1.0/R[0]) * R[3] * np.exp(2*np.pi*1j*freq[3]/16384)  # EP1
+        SUS_LINE1_REF_INVA_UIM_RESPRATIO = (
+            (1.0/(uim_actuation_epics[1]/daqdownsampling_response[1])) *
+            (1.0/R[0]) * R[1] * np.exp(2*np.pi*1j*freq[1]/16384))  # EP3
+        SUS_LINE2_REF_INVA_PUM_RESPRATIO = (
+            (1.0/(pum_actuation_epics[2]/daqdownsampling_response[2])) *
+            (1.0/R[0]) * R[2] * np.exp(2*np.pi*1j*freq[2]/16384))  # EP2
+        SUS_LINE3_REF_INVA_TST_RESPRATIO = (
+            (1.0/(tst_actuation_epics[3]/daqdownsampling_response[3])) *
+            (1.0/R[0]) * R[3] * np.exp(2*np.pi*1j*freq[3]/16384))  # EP1
 
         # Need to add a phase delay to the PCAL readback because the
         # PCAL channel acquires a phase delay from the model jump
         # to CAL-CS and the calculation expects no delay (as in GDS)
-        PCAL_LINE1_PCAL_DEMOD_PHASE = \
-            np.angle(np.exp(-2*np.pi*1j*freq[0]/16384), deg=True)
-        PCAL_LINE2_PCAL_DEMOD_PHASE = \
-            np.angle(np.exp(-2*np.pi*1j*freq[4]/16384), deg=True)
-        PCAL_LINE3_PCAL_DEMOD_PHASE = \
-            np.angle(np.exp(-2*np.pi*1j*freq[5]/16384), deg=True)
-        PCAL_LINE4_PCAL_DEMOD_PHASE = \
-            np.angle(np.exp(-2*np.pi*1j*freq[6]/16384), deg=True)
+        PCAL_LINE1_PCAL_DEMOD_PHASE = (
+            np.angle(np.exp(-2*np.pi*1j*freq[0]/16384), deg=True))
+        PCAL_LINE2_PCAL_DEMOD_PHASE = (
+            np.angle(np.exp(-2*np.pi*1j*freq[4]/16384), deg=True))
+        PCAL_LINE3_PCAL_DEMOD_PHASE = (
+            np.angle(np.exp(-2*np.pi*1j*freq[5]/16384), deg=True))
+        PCAL_LINE4_PCAL_DEMOD_PHASE = (
+            np.angle(np.exp(-2*np.pi*1j*freq[6]/16384), deg=True))
 
         # Need to add a phase delay to the SUS excitation because the
         # SUS channel acquires a phase delay from the model jump
         # to CAL-CS and the calculation expects no delay (as in GDS).
         # Note that if a replica SUS excitation is used in the CAL-CS model
         # then these should actually be set to zero.
-        SUS_LINE1_SUS_DEMOD_PHASE = \
-            np.angle(np.exp(-2*np.pi*1j*freq[1]/16384), deg=True)
-        SUS_LINE2_SUS_DEMOD_PHASE = \
-            np.angle(np.exp(-2*np.pi*1j*freq[2]/16384), deg=True)
-        SUS_LINE3_SUS_DEMOD_PHASE = \
-            np.angle(np.exp(-2*np.pi*1j*freq[3]/16384), deg=True)
+        SUS_LINE1_SUS_DEMOD_PHASE = (
+            np.angle(np.exp(-2*np.pi*1j*freq[1]/16384), deg=True))
+        SUS_LINE2_SUS_DEMOD_PHASE = (
+            np.angle(np.exp(-2*np.pi*1j*freq[2]/16384), deg=True))
+        SUS_LINE3_SUS_DEMOD_PHASE = (
+            np.angle(np.exp(-2*np.pi*1j*freq[3]/16384), deg=True))
 
         # Compute the sensing function without the optical response but need to
         # include the optical gain
@@ -261,16 +274,15 @@ class DARMModel(Model):
             self.sensing.detuned_spring_frequency,
             self.sensing.detuned_spring_q,
             pro_spring=self.sensing.is_pro_spring)
-        sensing_no_cavity_response = \
-            self.sensing.compute_sensing(freq) / \
-            signal.freqresp(coupled_cavity, 2.0*np.pi*freq)[1]
+        sensing_no_cavity_response = (
+            self.sensing.compute_sensing(freq) /
+            signal.freqresp(coupled_cavity, 2.0*np.pi*freq)[1])
         PCAL_LINE2_REF_C_NOCAVPOLE = sensing_no_cavity_response[4]  # EP4
         PCAL_LINE4_REF_C_NOCAVPOLE = sensing_no_cavity_response[6]  # EP10
         PCAL_LINE1_REF_C_NOCAVPOLE = sensing_no_cavity_response[0]  # EP19
 
         # DARM digital filtering
-        darm_digital_filter_response = \
-            self.digital.compute_response(freq)
+        darm_digital_filter_response = self.digital.compute_response(freq)
         PCAL_LINE2_REF_D = darm_digital_filter_response[4]  # EP5
         PCAL_LINE4_REF_D = darm_digital_filter_response[6]  # EP11
         PCAL_LINE1_REF_D = darm_digital_filter_response[0]  # EP20
@@ -439,6 +451,58 @@ class DARMModel(Model):
             'CAL-CS_TDEP_SUS_LINE3_SUS_DEMOD_PHASE': SUS_LINE3_SUS_DEMOD_PHASE,
         }
 
+    def write_epics_records(self, IFO, epics_output, filename, push_to_epics=False):
+        """
+        Push EPICS records to the front end CAL-CS model and save to a file
+
+        Parameters
+        ----------
+        IFO : str
+            'LHO' or 'LLO'
+        epics_output : `float`, dict
+            the output from compute_epics_records
+        filename : str
+            path and file name to be written out
+        push_to_epics : boolean
+            push epics records written to file to the front end epics records
+        """
+
+        with open(filename, 'w') as f:
+            for chan in sorted(epics_output.keys()):
+                chan_val = epics_output[chan]
+                if np.iscomplex(chan_val):
+                    chan = [f"{IFO}:{chan}_REAL", f"{IFO}:{chan}_IMAG"]
+                    chan_val = [np.real(chan_val), np.imag(chan_val)]
+                else:
+                    chan = [chan]
+                    chan_val = [chan_val]
+                for idx, ch in enumerate(chan):
+                    f.write(f"{ch} {chan_val[idx]:0.16e}\n")
+                    if push_to_epics:
+                        from ezca import ezca
+                        ezca = ezca.Ezca()
+                        ezca[ch] = '{:0.16e}'.format(chan_val[idx])
+        f.close()
+
+        # Ensure there is a moment for the EPICS records to get written
+        time.sleep(1)
+
+        if push_to_epics:
+            with open(filename, 'r') as f:
+                for chan in sorted(epics_output.keys()):
+                    chan_val = epics_output[chan]
+                    if np.iscomplex(chan_val):
+                        chan = [f"{IFO}:{chan}_REAL", f"{IFO}:{chan}_IMAG"]
+                        chan_val = [np.real(chan_val), np.imag(chan_val)]
+                    else:
+                        chan = [chan]
+                        chan_val = [chan_val]
+                    for idx, ch in enumerate(chan):
+                        readVal = ezca[ch]
+                        ratio = readVal / chan_val[idx]
+                    if abs(ratio[0] - 1) > 0.000001:
+                        print('Eeeeek! We had an EPICS connection error on '+chan+'. Check it!!')
+
     def plot(self, plot_selection='all', freq_min=0.1, freq_max=5000, filename=None, ifo='',
              label=None, style=None, ugf_start=10, ugf_end=1000, show=None, **kwargs):
         """
