@@ -21,7 +21,7 @@ import emcee
 import json
 from datetime import datetime
 from scipy.signal import freqresp, ZerosPolesGain
-from multiprocessing import Pool
+import multiprocessing as mp
 from scipy import signal
 
 import dttxml
@@ -203,10 +203,10 @@ class ProcessMeasurement(object):
     meas_type : `str`
         meas_type needs to be one of 'sensing', 'actuation_x_arm', or
         'actuation_y_arm' (needs to match config file section label)
-    meas_obj_1 : `Measurement`
+    meas_sensing_or_actuation : `Measurement`
         This is the object that contains the data for either the closed
         loop gain (sensing measurement) or suspension measurement (actuation)
-    meas_obj_2 : `Measurement`
+    meas_pcal_to_darm : `Measurement`
         This is the object that contains data for the PCAL to DARM TF
     meas_tup_1 : `str`, tuple
         This should be ('channel A', 'channel B') for the first measurement
@@ -223,7 +223,7 @@ class ProcessMeasurement(object):
     -------
     """
 
-    def __init__(self, config_file, meas_obj_1, meas_obj_2,
+    def __init__(self, config_file, meas_sensing_or_actuation, meas_pcal_to_darm,
                  meas_tup_1, meas_tup_2, meas1_cohThresh=0, meas2_cohThresh=0,
                  json_results_file=None):
         if len(meas_tup_1) != 2:
@@ -236,14 +236,19 @@ class ProcessMeasurement(object):
             raise ValueError('meas2_cohThresh must be in range [0,1]')
 
         self.PCAL = PcalModel(config_file)
-        self.measurement_1 = meas_obj_1
-        self.measurement_2 = meas_obj_2
+        self.config_file = os.path.abspath(config_file)
+        self.measurement_1 = meas_sensing_or_actuation
+        self.measurement_2 = meas_pcal_to_darm
         self.meas_tup_1 = meas_tup_1
         self.meas_tup_2 = meas_tup_2
         self.meas1_cohThresh = meas1_cohThresh
         self.meas2_cohThresh = meas2_cohThresh
         self.json_results_file = json_results_file
 
+        self.endstation = False
+        if 'CAL-DELTAL_REF_PCAL' not in self.meas_tup_2[0]:
+            self.endstation = True
+
     def pre_process_transfer_functions(self):
         """
         This method extracts the transfer function data using the channel names
@@ -308,7 +313,8 @@ class ProcessMeasurement(object):
                              'two transfer functions. Maybe check coherence',
                              ' thresholds.')
 
-        pcal_correction = self.PCAL.compute_pcal_correction(frequencies)
+        pcal_correction = self.PCAL.compute_pcal_correction(frequencies,
+                                                            self.endstation)
 
         processed_measurement_response_unc = np.sqrt(unc_meas1**2
                                                      + unc_meas2**2)
@@ -393,7 +399,9 @@ class ProcessMeasurement(object):
         xdata_gl, ydata_gl, yerr_gl = xdata, ydata, yerr
 
         # Use multiprocessing to run the MCMC faster
-        with Pool() as pool:
+        # Using this instead of "with Pool() as pool:"
+        # See pyDARM issue #47 for the reason for this change
+        with mp.get_context('fork').Pool() as pool:
             sampler = emcee.EnsembleSampler(len(priors[:, 0]),
                                             len(priors[0, :]),
                                             _ln_prob,
@@ -437,7 +445,7 @@ class ProcessMeasurement(object):
         mcmc_map_vals : `float`, array-like
             array of maximum a postiori values found from the MCMC
         measurement : str
-            measurement type; one of 'sensing' or 'actuation'
+            measurement type; one of 'sensing', 'actuation_x_arm', or 'actuation_y_arm'
 
         Returns
         -------
@@ -447,16 +455,20 @@ class ProcessMeasurement(object):
 
         if measurement == 'sensing':
             config_dict = self.sensing.config_to_dict()
-        elif measurement == 'actuation':
+        elif (measurement == 'actuation_x_arm' or measurement == 'actuation_y_arm'):
             config_dict = self.actuation.config_to_dict()
         else:
             raise ValueError('Neither actuation nor sensing was provided')
 
         measurement_info = {'Measurement analysis {}'.format(
             now.strftime('%d/%m/%Y %H:%M:%S')): {
+            'model_config_file': self.config_file,
+            'meas_type': measurement,
             'model_config': config_dict,
             'loop_meas': self.measurement_1.filename,
             'pcal_meas': self.measurement_2.filename,
+            'loop_tup': self.meas_tup_1,
+            'pcal_tup': self.meas_tup_2,
             'loop_cohthresh': self.meas1_cohThresh,
             'pcal_cohthresh': self.meas2_cohThresh,
             'fmin': fmin, 'fmax': fmax,
@@ -591,13 +603,13 @@ class ProcessMeasurement(object):
                                is_pro_spring])
 
             if query_match[21:] == match:
-                output = output[len(output)-1]
+                output = output[-1]
                 break
             if match == 'first' and len(output) != 0:
                 break
 
         if match == 'latest':
-            output = output[len(output)-1]
+            output = output[-1]
 
         if match != 'latest' and match != 'first' and match != 'all' and query_match[21:] != match:
             raise ValueError(
@@ -653,6 +665,15 @@ class ProcessMeasurement(object):
         y_pred, cov = gp.predict(np.log10(freq[None, :].T), return_cov=True)
         y_pred = y_pred[:, 0] + 1j*y_pred[:, 1]
 
+        # Remove 2nd feature standard deviation and covariance contribution.
+        # Since the GP fits real and imaginary as two features with the identical
+        # uncertainties, the inferred covariance matrices have identical structure.
+        # So we need to simply throw the second feature's identical uncertainty away.
+        # This requires scikit-learn > 1.0.2.
+
+        sigma = sigma[:, 0]
+        cov = cov[:, :, 0]
+
         return y_pred, sigma, cov
 
 
@@ -667,9 +688,9 @@ class ProcessSensingMeasurement(ProcessMeasurement):
     ----------
     config_file : `str`
         This is the *.ini model file to be used to process the measurement data
-    meas_obj_1 : `Measurement`
+    meas_clg : `Measurement`
         This is the object that contains the data for the closed loop gain
-    meas_obj_2 : `Measurement`
+    meas_pcal_to_darm : `Measurement`
         This is the object that contains data for the PCAL to DARM TF
     meas_tup_1 : `str`, tuple
         This should be ('channel A', 'channel B') for the first measurement
@@ -688,10 +709,10 @@ class ProcessSensingMeasurement(ProcessMeasurement):
     -------
     """
 
-    def __init__(self, config_file, meas_obj_1, meas_obj_2, meas_tup_1,
+    def __init__(self, config_file, meas_clg, meas_pcal_to_darm, meas_tup_1,
                  meas_tup_2, meas1_cohThresh=0, meas2_cohThresh=0,
                  json_results_file=None):
-        super().__init__(config_file, meas_obj_1, meas_obj_2,
+        super().__init__(config_file, meas_clg, meas_pcal_to_darm,
                          meas_tup_1, meas_tup_2, meas1_cohThresh,
                          meas2_cohThresh, json_results_file)
         self.sensing = SensingModel(config_file)
@@ -724,7 +745,7 @@ class ProcessSensingMeasurement(ProcessMeasurement):
 
         # For the sensing function correction, we need the known terms
         tf_meas1_correction = \
-            self.sensing.known_sensing_terms_for_measurement(frequencies)
+            self.sensing.sensing_residual(frequencies)
 
         # Now bring everything together; see G1501518
         processed_measurement_response = \
@@ -845,7 +866,7 @@ class ProcessSensingMeasurement(ProcessMeasurement):
 
         if save_to_file is not None:
             self.save_results_to_json(
-                save_to_file, fmin, fmax, np.median(chain, axis=0), 'sensing')
+                save_to_file, fmin, fmax, np.median(chain, axis=0), self.meas_type)
 
         return chain
 
@@ -987,7 +1008,7 @@ class ProcessSensingMeasurement(ProcessMeasurement):
 
         output = []
 
-        ref_date, ref_map, ref_spring = self.query_results_from_json('sensing', fmin,
+        ref_date, ref_map, ref_spring = self.query_results_from_json(self.meas_type, fmin,
                                                                      fmax)
 
         for idx, meas in enumerate(measurement_list):
@@ -995,16 +1016,16 @@ class ProcessSensingMeasurement(ProcessMeasurement):
                 meas.get_processed_measurement_response()
             if fmin_list is None and fmax_list is None:
                 this_date, this_map, this_spring = meas.query_results_from_json(
-                    'sensing')
+                    self.meas_type)
             elif fmin_list is not None and fmax_list is None:
                 this_date, this_map, this_spring = meas.query_results_from_json(
-                    'sensing', fmin=fmin_list[idx])
+                    self.meas_type, fmin=fmin_list[idx])
             elif fmin_list is None and fmax_list is not None:
                 this_date, this_map, this_spring = meas.query_results_from_json(
-                    'sensing', fmax=fmax_list[idx])
+                    self.meas_type, fmax=fmax_list[idx])
             else:
                 this_date, this_map, this_spring = meas.query_results_from_json(
-                    'sensing', fmin_list[idx], fmax_list[idx])
+                    self.meas_type, fmin_list[idx], fmax_list[idx])
 
             # We want to compute the transfer function as though we were
             # measuring the detector at the same reference time, so we need
@@ -1076,7 +1097,7 @@ class ProcessSensingMeasurement(ProcessMeasurement):
                 (len(measurement_list) == len(fmax_list) and
                  len(fmin_list) == 0))
 
-        ref_date, MAP, ref_spring = self.query_results_from_json('sensing', fmin,
+        ref_date, MAP, ref_spring = self.query_results_from_json(self.meas_type, fmin,
                                                                  fmax)
 
         stacked_meas = self.stack_measurements(measurement_list, fmin, fmax,
@@ -1124,10 +1145,10 @@ class ProcessActuationMeasurement(ProcessMeasurement):
         This is the *.ini model file to be used to process the measurement data
     meas_type : `str`
         This should be one of 'actuation_x_arm' or 'actuation_y_arm'
-    meas_obj_1 : `Measurement`
+    meas_sus_exc_to_darm : `Measurement`
         This is the object that contains the data for the DARM_IN channel to
         suspension excitation response
-    meas_obj_2 : `Measurement`
+    meas_pcal_to_darm : `Measurement`
         This is the object that contains data for the PCAL to DARM TF
     meas_tup_1 : `str`, tuple
         This should be ('channel A', 'channel B') for the first measurement
@@ -1146,13 +1167,13 @@ class ProcessActuationMeasurement(ProcessMeasurement):
     -------
     """
 
-    def __init__(self, config_file, meas_type, meas_obj_1, meas_obj_2,
+    def __init__(self, config_file, meas_type, meas_sus_exc_to_darm, meas_pcal_to_darm,
                  meas_tup_1, meas_tup_2, meas1_cohThresh=0, meas2_cohThresh=0,
                  json_results_file=None):
         if not (meas_type == 'actuation_x_arm' or
                 meas_type == 'actuation_y_arm'):
             raise ValueError('Invalid meas_type')
-        super().__init__(config_file, meas_obj_1, meas_obj_2,
+        super().__init__(config_file, meas_sus_exc_to_darm, meas_pcal_to_darm,
                          meas_tup_1, meas_tup_2, meas1_cohThresh,
                          meas2_cohThresh, json_results_file)
         self.actuation = ActuationModel(config_file, measurement=meas_type)
@@ -1276,7 +1297,7 @@ class ProcessActuationMeasurement(ProcessMeasurement):
 
         if save_to_file is not None:
             self.save_results_to_json(
-                save_to_file, fmin, fmax, np.median(chain, axis=0), 'actuation')
+                save_to_file, fmin, fmax, np.median(chain, axis=0), self.meas_type)
 
         return chain
 
