@@ -339,12 +339,10 @@ def residuals(freq_model, tf_model, freq_data, tf_data, unc_data,
          freq_min=freq_data[0], freq_max=freq_data[-1],
          title='Measurement vs Model')
     """
-    f_mag_model = interp1d(freq_model, np.abs(tf_model), kind='cubic')
-    f_phase_model = interp1d(freq_model, np.angle(tf_model), kind='cubic')
-    residuals_mag = np.abs(tf_data)/f_mag_model(freq_data)
-    residuals_phase = np.angle(tf_data)-f_phase_model(freq_data)
-    residuals_tf = residuals_mag*np.exp(1j*residuals_phase)
-    residuals_tuple = freq_data, residuals_tf, unc_data
+    # First interpolate the model tf at the frequencies of the data points
+    tf_model_interp = np.interp(freq_data, freq_model, tf_model)
+    tf_ratio = tf_data/tf_model_interp
+    tf_ratio_tuple = freq_data, tf_ratio, unc_data
 
     if plot_input:
         if filename is not None:
@@ -354,7 +352,7 @@ def residuals(freq_model, tf_model, freq_data, tf_data, unc_data,
         plot(freq_model, tf_model, (freq_data, tf_data, unc_data), title='Measurement vs Model',
              freq_min=freq_data[0], freq_max=freq_data[-1], filename=filename_input)
 
-    bode_handle = plot(residuals_tuple, freq_min=freq_data[0], freq_max=freq_data[-1],
+    bode_handle = plot(tf_ratio_tuple, freq_min=freq_data[0], freq_max=freq_data[-1],
                        mag_min=mag_min, mag_max=mag_max, phase_min=phase_min,
                        phase_max=phase_max, show=show, title='Measurement vs Model Residuals')
     bode_handle.ax_mag.set_yscale('linear')
