@@ -18,8 +18,7 @@ import numpy as np
 from scipy import signal, io, constants
 from collections import namedtuple
 
-from .utils import (serielZPK, digital_delay_filter, load_foton_export_tf,
-                    compute_digital_filter_response)
+from .utils import serielZPK, compute_digital_filter_response
 from .digital import iopdownsamplingfilters
 from .model import Model
 
@@ -91,9 +90,10 @@ class SensingModel(Model):
         coupled_cavity = serielZPK(cavity_pole, detuning)
         return coupled_cavity
 
-    def omc_analog_dcpd_readout_response(self, name, frequencies):
+    def omc_dcpd_transimpedence_amplifier_response(self, name, frequencies):
         """
-        The transfer function of the analog OMC DCPD readout electronics
+        The transfer function of the analog OMC DCPD transimpedence amplifier
+        electronics
 
         Parameters
         ----------
@@ -107,7 +107,8 @@ class SensingModel(Model):
         Returns
         -------
         response : `complex128`, array-like
-            transfer function response of the analog electronics in path `name`
+            transfer function response of the analog transimpedence amplifier
+            in path `name`
         """
 
         # Make sure that name is listed uniquely in self.omc_path_names
@@ -140,7 +141,32 @@ class SensingModel(Model):
             omc_trans_amplifier_response = np.ones(len(frequencies),
                                                    dtype='complex128')
 
-        # Whitening filter
+        return omc_trans_amplifier_response
+
+    def omc_dcpd_whitening_response(self, name, frequencies):
+        """
+        The transfer function of the analog OMC DCPD whitening electronics
+
+        Parameters
+        ----------
+        name : str
+            a string (e.g., 'A' or 'A_A') indicating the OMC path name to be
+            evaluated. This string should be listed in
+            SensingModel.omc_path_names variable
+        frequencies : `float`, array-like
+            array of frequencies to compute the response
+
+        Returns
+        -------
+        response : `complex128`, array-like
+            transfer function response of the analog whitening electronics in
+            path `name`
+        """
+
+        # Make sure that name is listed uniquely in self.omc_path_names
+        idx = self.omc_path_names.index(name)
+        assert len(set(self.omc_path_names)) == len(self.omc_path_names)
+
         # The zeros and poles need to have the name appended to them
         whitening_name = getattr(self, 'whitening_mode_names')[idx]
         if (hasattr(self, f'omc_meas_z_whitening_{whitening_name}') and
@@ -168,6 +194,39 @@ class SensingModel(Model):
             omc_whitening_response = np.ones(len(frequencies),
                                              dtype='complex128')
 
+        return omc_whitening_response
+
+    def omc_analog_dcpd_readout_response(self, name, frequencies):
+        """
+        The transfer function of the analog OMC DCPD readout electronics
+
+        Parameters
+        ----------
+        name : str
+            a string (e.g., 'A' or 'A_A') indicating the OMC path name to be
+            evaluated. This string should be listed in
+            SensingModel.omc_path_names variable
+        frequencies : `float`, array-like
+            array of frequencies to compute the response
+
+        Returns
+        -------
+        response : `complex128`, array-like
+            transfer function response of the analog electronics in path `name`
+        """
+
+        # Make sure that name is listed uniquely in self.omc_path_names
+        idx = self.omc_path_names.index(name)
+        assert len(set(self.omc_path_names)) == len(self.omc_path_names)
+
+        # First the transimpedence amplifier is computed
+        omc_trans_amplifier_response = (
+            self.omc_dcpd_transimpedence_amplifier_response(name, frequencies))
+
+        # Whitening filter
+        omc_whitening_response = (
+            self.omc_dcpd_whitening_response(name, frequencies))
+
         # super high frequency poles look like a delay (see G2200551)
         super_high_frequency_poles_apparent_delay = np.exp(
             -2.0*np.pi*1j *
@@ -376,40 +435,31 @@ class SensingModel(Model):
                       self.single_pole_approximation_delay_correction *
                       frequencies)
 
-    def known_sensing_terms_for_measurement(self, frequencies):
+    def sensing_residual(self, frequencies):
         """
-        Compute the known terms in the sensing measurement that we account
-        for by dividing out. See G1501518-v21
+        Compute the residual sensing function
 
-        Note that we are now allocating all gain on the sensing path to
-        the overall optical gain (previously a 0.99 gain on AA was not
-        allocated to the optical gain)
+        This is everything in the sensing function except the optical gain
+        and the optical response
 
         Parameters
         ----------
         frequencies : `float`, array-like
-            array of frequencies to compute the known sensing terms
+            array of frequencies to compute the residual sensing function
 
         Returns
         -------
-        sensing_correction : `complex128`, array-like
-            correction terms
+        C_res : `complex128`, array-like
+            transfer function response
         """
 
-        light_travel_time_delay = \
-            self.light_travel_time_delay_response(frequencies)
-        single_pole_approximation_delay_correction = \
-            self.single_pole_approximation_delay_correction_response(
-                frequencies)
-        omc_paths_response = self.omc_combine_path_responses(frequencies)
+        C_res = (self.sensing_sign *
+                 self.light_travel_time_delay_response(frequencies) *
+                 self.single_pole_approximation_delay_correction_response(
+                     frequencies) *
+                 self.omc_combine_path_responses(frequencies))
 
-        sensing_correction = \
-            self.sensing_sign * \
-            light_travel_time_delay * \
-            single_pole_approximation_delay_correction * \
-            omc_paths_response
-
-        return sensing_correction
+        return C_res
 
     def compute_sensing(self, frequencies):
         """
@@ -436,18 +486,8 @@ class SensingModel(Model):
         # By design there are no digital delays to the OMC user model
         # see G1601472
 
-        # light travel time delay
-        light_travel_time_delay = \
-            self.light_travel_time_delay_response(frequencies)
-
-        # single pole approximation needs a correction for too much delay
-        # see G1501316
-        single_pole_approximation_delay_correction = \
-            self.single_pole_approximation_delay_correction_response(
-                frequencies)
-
-        # OMC paths response
-        omc_paths_response = self.omc_combine_path_responses(frequencies)
+        # sensing residual (all other terms) except for the overall gain
+        C_res = self.sensing_residual(frequencies)
 
         # All together:
         # sensing sign
@@ -456,98 +496,12 @@ class SensingModel(Model):
         # light travel time delay
         # single pole approximation delay correction
         # OMC DCPD electronics paths
-        tf = self.sensing_sign * \
-            self.coupled_cavity_optical_gain * \
-            coupled_cavity_filter_response * \
-            light_travel_time_delay * \
-            single_pole_approximation_delay_correction * \
-            omc_paths_response
+        tf = (self.coupled_cavity_optical_gain *
+              coupled_cavity_filter_response *
+              C_res)
 
         return tf
 
-    def sensing_residual(self, frequencies):
-        """
-        Compute the residual sensing function
-
-        This is everything in the sensing function except the optical gain
-        and the optical response
-
-        Parameters
-        ----------
-        frequencies : `float`, array-like
-            array of frequencies to compute displacement to CAL-CW residual
-
-        Returns
-        -------
-        C_res : `complex128`, array-like
-            transfer function response
-        """
-
-        coupled_cavity = self.optical_response(
-            self.coupled_cavity_pole_frequency,
-            self.detuned_spring_frequency,
-            self.detuned_spring_q,
-            pro_spring=self.is_pro_spring)
-        C_res = (self.compute_sensing(frequencies) / (
-            self.coupled_cavity_optical_gain *
-            signal.freqresp(coupled_cavity, 2.0*np.pi*frequencies)[1]))
-
-        return C_res
-
-    def displacement_to_calcs_residual(self, frequencies):
-        """
-        Compute the ETM displacement to CAL-CS residual
-
-        This will be C * 1 16k clock cycle delay * foton inverse sensing filter
-
-        Parameters
-        ----------
-        frequencies : `float`, array-like
-            array of frequencies to compute displacement to CAL-CW residual
-
-        Returns
-        -------
-        tf : `complex128`, array-like
-            transfer function response
-        """
-
-        # Get the sensing model
-        C_pydarm = self.compute_sensing(frequencies)
-
-        # Need the coupled cavity model
-        coupled_cavity = self.optical_response(
-            self.coupled_cavity_pole_frequency, self.detuned_spring_frequency,
-            self.detuned_spring_q, pro_spring=self.is_pro_spring)
-
-        # Need one 16k clock cycle delay going from OMC to CAL-CS
-        one_clock_cycle_filter = digital_delay_filter(1, 2**14)
-        one_clock_cycle_filter_response = \
-            signal.dfreqresp(one_clock_cycle_filter,
-                             2.0*np.pi*frequencies/2**14)[1]
-
-        # load inverse sensing data from foton file export (1/SRC_D2N)
-        if hasattr(self, 'gds_c_foton_invsensing_tf') and \
-                self.gds_c_foton_invsensing_tf != '':
-            [foton_export_freq, foton_export_tf] = \
-                load_foton_export_tf(
-                    self.dpath(self.gds_c_foton_invsensing_tf))
-        else:
-            # This is the fake if no foton export file is provided
-            foton_export_freq = frequencies
-            foton_export_tf = 1.0 / \
-                signal.freqresp(
-                    coupled_cavity,
-                    2.0*np.pi*foton_export_freq)[1]
-
-        # interpolate the foton filter data to the frequencies used
-        foton_inv_sensing_interp = np.interp(frequencies,
-                                             foton_export_freq,
-                                             foton_export_tf)
-
-        return C_pydarm * \
-            one_clock_cycle_filter_response * \
-            foton_inv_sensing_interp
-
     def old_school_senspars(self, frequencies):
         """
         The old version of pyDARM stored intermediate data products in a
