@@ -29,20 +29,34 @@ from .model import isfloat
 
 class DARMUncertainty(object):
 
-    def __init__(self, config):
+    def __init__(self, config, uncertainty_config=None):
         """
+        Initialize a DARMUncertainty object
+
+        Note that any string or path-to-file string in `uncertainty_config`
+        will override any uncertainty key value parameters in the `config`
+        string or path-to-file
+
         Parameters
         ----------
-        Returns
-        -------
+        config : file path or string
+            INI config
+        uncertainty_config : file path or string, optional
+            INI config
         """
 
-        # Load configuration and set attributes of DARMUncertainty object
+        # Load configuration and set attributes of DARMUncertainty object.
+        # Initialize and compute derived data products from configuration.
+        # The default is one ini file containing the DARM model parameters and
+        # the uncertainty parameters. Alternatively, the uncertainty parameters
+        # can be in a separate file from the DARM model parameters. The
+        # optional uncertainty will override anything in config
         self._config = None
         self._load_configuration(config)
+        self.darm = DARMModel(config)
 
-        # Initialize and compute derived data products from configuration
-        self.darm = DARMModel(self.model)
+        if uncertainty_config is not None:
+            self._load_configuration(uncertainty_config)
 
         # Set the sensing mcmc chain or None
         if hasattr(self, 'sensing_mcmc_file') and self.sensing_mcmc_file != '':
