@@ -460,7 +460,7 @@ def read_filter_file(filename):
                     fname = line_el[2]
                     index = int(line_el[3])
                     design_str = line_el[4]
-                    while line_el[len(line_el)-1] == '\\':
+                    while line_el[-1] == '\\':
                         line = next(file)
                         line_el = line.split()
                         if len(line_el) == 1:
