@@ -613,48 +613,7 @@ class TestDrivealignToDisplacement(unittest.TestCase):
 
     def setUp(self):
         self.frequencies = np.logspace(0, np.log10(5000.), 10)
-        self.TST_known_tf = np.array(
-            [-2.222881500981626e-15+3.691011841669097e-16j,
-             1.1764193182970055e-15+2.5849635956379406e-16j,
-             3.1617534435707367e-16-9.92839053775603e-16j,
-             -1.1700305084971196e-16-2.1853458973419276e-16j,
-             -5.006230661758991e-17-2.4902470964105e-17j,
-             -9.308653195938007e-18+1.9222273202986028e-19j,
-             -1.1111368291657706e-18+5.936117452101705e-19j,
-             -5.412995749774128e-20+1.7611360028653088e-19j,
-             2.4897361921808887e-20-3.511338429355629e-21j,
-             1.3890965613664794e-21+1.7923601565333542e-21j])
-        self.PUM_known_tf = np.array(
-            [-5.636311671296026e-13+1.1503626120207196e-12j,
-             6.539680039025585e-15+3.344573000972991e-16j,
-             9.436017253315024e-17-3.810611139309467e-17j,
-             1.2201453005199638e-18-1.1859853195589023e-18j,
-             2.3160192732375865e-21-2.230152515596516e-20j,
-             -1.7088823698982324e-22-1.4723824048886304e-22j,
-             -3.9895135060052445e-25+2.9092246327605506e-25j,
-             1.1353220905153887e-26+8.382906433078052e-27j,
-             3.841572759246625e-27-3.1425560227349834e-27j,
-             -4.588083981792824e-30-2.906408489212469e-30j])
-        self.UIM_known_tf = np.array(
-            [-3.430763161527003e-11+1.2696021299733796e-10j,
-             -2.4899857500856507e-13-1.547223638897765e-13j,
-             -8.152763544441236e-16+7.476151512063072e-18j,
-             -2.3086963982589343e-18+5.500858196896584e-20j,
-             -8.663921746539537e-21+5.386329216110624e-22j,
-             5.290544966557914e-23+1.2746440643491213e-24j,
-             2.154672382803322e-24-9.81407533663976e-25j,
-             3.672997786320173e-29-5.963176342634914e-29j,
-             1.863046548604978e-32+9.566192650734058e-33j,
-             -2.0460864930999043e-37-3.153188911481507e-39j])
-
-    def tearDown(self):
-        del self.frequencies
-        del self.TST_known_tf
-        del self.PUM_known_tf
-        del self.UIM_known_tf
-
-    def test_drivealign_to_displacement(self):
-        A = pydarm.actuation.ActuationModel('''
+        self.model_string = '''
 [metadata]
 [interferometer]
 [actuation_x_arm]
@@ -692,26 +651,101 @@ pum_drive_align_gain = 1.0
 uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
 uim_drive_align_modules =
 uim_drive_align_gain = 1.0
-''',
+'''
+        self.sign_string = "darm_feedback_sign = -1"
+        self.TST_known_long_tf = np.array(
+            [-2.222881500981626e-15+3.691011841669097e-16j,
+             1.1764193182970055e-15+2.5849635956379406e-16j,
+             3.1617534435707367e-16-9.92839053775603e-16j,
+             -1.1700305084971196e-16-2.1853458973419276e-16j,
+             -5.006230661758991e-17-2.4902470964105e-17j,
+             -9.308653195938007e-18+1.9222273202986028e-19j,
+             -1.1111368291657706e-18+5.936117452101705e-19j,
+             -5.412995749774128e-20+1.7611360028653088e-19j,
+             2.4897361921808887e-20-3.511338429355629e-21j,
+             1.3890965613664794e-21+1.7923601565333542e-21j])
+        self.PUM_known_long_tf = np.array(
+            [-5.636311671296026e-13+1.1503626120207196e-12j,
+             6.539680039025585e-15+3.344573000972991e-16j,
+             9.436017253315024e-17-3.810611139309467e-17j,
+             1.2201453005199638e-18-1.1859853195589023e-18j,
+             2.3160192732375865e-21-2.230152515596516e-20j,
+             -1.7088823698982324e-22-1.4723824048886304e-22j,
+             -3.9895135060052445e-25+2.9092246327605506e-25j,
+             1.1353220905153887e-26+8.382906433078052e-27j,
+             3.841572759246625e-27-3.1425560227349834e-27j,
+             -4.588083981792824e-30-2.906408489212469e-30j])
+        self.UIM_known_long_tf = np.array(
+            [-3.430763161527003e-11+1.2696021299733796e-10j,
+             -2.4899857500856507e-13-1.547223638897765e-13j,
+             -8.152763544441236e-16+7.476151512063072e-18j,
+             -2.3086963982589343e-18+5.500858196896584e-20j,
+             -8.663921746539537e-21+5.386329216110624e-22j,
+             5.290544966557914e-23+1.2746440643491213e-24j,
+             2.154672382803322e-24-9.81407533663976e-25j,
+             3.672997786320173e-29-5.963176342634914e-29j,
+             1.863046548604978e-32+9.566192650734058e-33j,
+             -2.0460864930999043e-37-3.153188911481507e-39j])
+        signval = float(self.sign_string.split(' = ')[-1])
+        self.TST_known_darm_tf = signval * self.TST_known_long_tf
+        self.PUM_known_darm_tf = signval * self.PUM_known_long_tf
+        self.UIM_known_darm_tf = signval * self.UIM_known_long_tf
+
+    def tearDown(self):
+        del self.frequencies
+        del self.model_string
+        del self.sign_string
+        del self.TST_known_long_tf
+        del self.PUM_known_long_tf
+        del self.UIM_known_long_tf
+        del self.TST_known_darm_tf
+        del self.PUM_known_darm_tf
+        del self.UIM_known_darm_tf
+
+    def test_drivealign_to_longitudinal_displacement(self):
+        A = pydarm.actuation.ActuationModel(self.model_string,
                                             measurement='actuation_x_arm')
         [uim_response, pum_response, tst_response] = \
-            A.drivealign_to_displacement(self.frequencies)
+            A.drivealign_to_longitudinal_displacement(self.frequencies)
         for n in range(len(self.frequencies)):
             self.assertAlmostEqual(np.abs(uim_response[n]),
-                                   np.abs(self.UIM_known_tf[n]))
+                                   np.abs(self.UIM_known_long_tf[n]))
             self.assertAlmostEqual(
                 np.angle(uim_response[n], deg=True),
-                np.angle(self.UIM_known_tf[n], deg=True))
+                np.angle(self.UIM_known_long_tf[n], deg=True))
             self.assertAlmostEqual(np.abs(pum_response[n]),
-                                   np.abs(self.PUM_known_tf[n]))
+                                   np.abs(self.PUM_known_long_tf[n]))
             self.assertAlmostEqual(
                 np.angle(pum_response[n], deg=True),
-                np.angle(self.PUM_known_tf[n], deg=True))
+                np.angle(self.PUM_known_long_tf[n], deg=True))
             self.assertAlmostEqual(np.abs(tst_response[n]),
-                                   np.abs(self.TST_known_tf[n]))
+                                   np.abs(self.TST_known_long_tf[n]))
             self.assertAlmostEqual(
                 np.angle(tst_response[n], deg=True),
-                np.angle(self.TST_known_tf[n], deg=True))
+                np.angle(self.TST_known_long_tf[n], deg=True))
+
+    def test_drivealign_to_darm_displacement(self):
+        A = pydarm.actuation.ActuationModel(
+            self.model_string + self.sign_string,
+            measurement='actuation_x_arm')
+        [uim_response, pum_response, tst_response] = \
+            A.drivealign_to_darm_displacement(self.frequencies)
+        for n in range(len(self.frequencies)):
+            self.assertAlmostEqual(np.abs(uim_response[n]),
+                                   np.abs(self.UIM_known_darm_tf[n]))
+            self.assertAlmostEqual(
+                np.angle(uim_response[n], deg=True),
+                np.angle(self.UIM_known_darm_tf[n], deg=True))
+            self.assertAlmostEqual(np.abs(pum_response[n]),
+                                   np.abs(self.PUM_known_darm_tf[n]))
+            self.assertAlmostEqual(
+                np.angle(pum_response[n], deg=True),
+                np.angle(self.PUM_known_darm_tf[n], deg=True))
+            self.assertAlmostEqual(np.abs(tst_response[n]),
+                                   np.abs(self.TST_known_darm_tf[n]))
+            self.assertAlmostEqual(
+                np.angle(tst_response[n], deg=True),
+                np.angle(self.TST_known_darm_tf[n], deg=True))
 
 
 class TestDrivealignToDisplacementForMeasurement(unittest.TestCase):
@@ -826,16 +860,16 @@ class TestComputeActuationSingleStage(unittest.TestCase):
         # TST stage
         self.frequencies = np.logspace(0, np.log10(5000.), 10)
         self.known_single_stage = np.array(
-            [-2.2226292345691665e-15+3.7055835896161056e-16j,
-             1.1768496904693823e-15+2.5650581888551325e-16j,
-             3.1184631196028356e-16-9.942029247767746e-16j,
-             -1.1944786777482867e-16-2.1720731539345392e-16j,
-             -5.076181343593909e-17-2.344496830068076e-17j,
-             -9.269325912589879e-18+8.84859270770252e-19j,
-             -9.775946787962668e-19+7.95794257011437e-19j,
-             3.8327563676853116e-20+1.8108722537309245e-19j,
-             -7.300815076923215e-21-2.6726320383495696e-20j,
-             2.3885715105224723e-21-3.8671879493130684e-22j])
+            [-2.2227711784771e-15+3.6970596409765503e-16j,
+             1.176595688578433e-15+2.5766842006473456e-16j,
+             3.1437592695198093e-16-9.934059363981118e-16j,
+             -1.1802093456019643e-16-2.1798593987209916e-16j,
+             -5.035849592429691e-17-2.4299179619229708e-17j,
+             -9.299048831148297e-18+4.806970343138081e-19j,
+             -1.0605041246314987e-18+6.814035934325136e-19j,
+             -1.4850855646504008e-20+1.84502133909869e-19j,
+             1.2735108604970117e-20-2.4605184633493507e-20j,
+             -4.47870011782826e-22+2.3778641129758554e-21j])
 
     def tearDown(self):
         del self.frequencies
@@ -905,6 +939,89 @@ uim_drive_align_gain = 1.0
                 np.angle(self.known_single_stage[n], deg=True))
 
 
+class TestActuationStageResidual(unittest.TestCase):
+
+    def setUp(self):
+        self.frequencies = np.logspace(0, np.log10(5000.), 10)
+        self.known_residual = np.array(
+            [0.9902575615036632-0.0016871227384657537j,
+             0.9902492531426801-0.004346523822516416j,
+             0.9901941084811342-0.011197736013719756j,
+             0.9898281244071855-0.028844992443191404j,
+             0.9874002528379973-0.07424928569758847j,
+             0.9713421858209306-0.19019480399674393j,
+             0.8672208231254349-0.47157796687119996j,
+             0.27670562456405506-0.9299571986485128j,
+             -0.8694694693325025+0.1259688027976555j,
+             -0.32255394920871006-0.4152917257163378j])
+
+    def tearDown(self):
+        del self.frequencies
+        del self.known_residual
+
+    def test_actuation_stage_residual(self):
+        A = pydarm.actuation.ActuationModel('''
+[metadata]
+[interferometer]
+[actuation_x_arm]
+darm_feedback_sign = -1
+tst_NpV2 = 4.427e-11
+linearization = OFF
+actuation_esd_bias_voltage = -9.3
+sus_filter_file = test/H1SUSETMX_1236641144.txt
+tst_isc_inf_bank = ETMX_L3_ISCINF_L
+tst_isc_inf_modules =
+tst_isc_inf_gain = 1.0
+tst_lock_bank = ETMX_L3_LOCK_L
+tst_lock_modules = 5,8,9,10
+tst_lock_gain = 1.0
+tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
+tst_drive_align_modules = 4,5
+tst_drive_align_gain = -35.7
+suspension_file = test/H1susdata_O3.mat
+tst_driver_meas_Z_UL = 129.7e3
+tst_driver_meas_Z_LL = 90.74e3
+tst_driver_meas_Z_UR = 93.52e3
+tst_driver_meas_Z_LR = 131.5e3
+tst_driver_meas_P_UL = 3.213e3, 31.5e3
+tst_driver_meas_P_LL = 3.177e3, 26.7e3
+tst_driver_meas_P_UR = 3.279e3, 26.6e3
+tst_driver_meas_P_LR = 3.238e3, 31.6e3
+tst_driver_DC_gain_VpV_HV = 40
+tst_driver_DC_gain_VpV_LV = 1.881
+anti_imaging_rate_string = 16k
+anti_imaging_method = biquad
+analog_anti_imaging_file = test/H1aa.mat
+dac_gain = 7.62939453125e-05
+unknown_actuation_delay = 15e-6
+pum_driver_DC_trans_ApV = 2.6847e-4
+pum_coil_outf_signflip = 1
+pum_NpA = 0.02947
+uim_driver_DC_trans_ApV = 6.1535e-4
+uim_NpA = 1.634
+pum_lock_bank = ETMX_L2_LOCK_L
+pum_lock_modules = 7
+pum_lock_gain = 23.0
+pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
+pum_drive_align_modules = 6,7
+pum_drive_align_gain = 1.0
+uim_lock_bank = ETMX_L1_LOCK_L
+uim_lock_modules = 10
+uim_lock_gain = 1.06
+uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
+uim_drive_align_modules =
+uim_drive_align_gain = 1.0
+''',
+                                            measurement='actuation_x_arm')
+        residual = A.actuation_stage_residual(self.frequencies)
+        for n in range(len(self.frequencies)):
+            self.assertAlmostEqual(np.abs(residual[n]),
+                                   np.abs(self.known_residual[n]))
+            self.assertAlmostEqual(
+                np.angle(residual[n], deg=True),
+                np.angle(self.known_residual[n], deg=True))
+
+
 class TestDARMOutputMatrixValues(unittest.TestCase):
 
     def setUp(self):
