@@ -447,7 +447,7 @@ digital_filter_gain = 400,1
         test_epics = darm.compute_epics_records(
             self.frequencies[0], self.frequencies[1], self.frequencies[2],
             self.frequencies[3], self.frequencies[4], self.frequencies[5],
-            self.frequencies[6], arm='x')
+            self.frequencies[6], arm='x', endstation=True)
         for idx, (key, val) in enumerate(self.known_epics.items()):
             self.assertAlmostEqual(
                 np.abs(test_epics[key]/self.known_epics[key]), 1.0, places=6)
