@@ -1467,7 +1467,7 @@ class TestActuationSaveResultsToJson(unittest.TestCase):
             ('H1:CAL-PCALY_RX_PD_OUT_DQ', 'H1:LSC-DARM_IN1_DQ'))
 
         meas.save_results_to_json(self.filename, self.fmin, self.fmax,
-                                  self.mcmc_map_vals, 'actuation')
+                                  self.mcmc_map_vals, 'actuation_x_arm')
 
 
 class TestActuationQueryResultsFromJson(unittest.TestCase):
