@@ -43,6 +43,85 @@ class TestOpticalResponse(unittest.TestCase):
                 np.angle(self.known_optical_response[n], deg=True))
 
 
+class TestOMCDCPDTransimpedenceAmplifierResponse(unittest.TestCase):
+
+    def setUp(self):
+        self.frequencies = np.logspace(0, np.log10(5000.), 10)
+        self.known_response = np.array(
+            [0.9999999874151851-0.0001291724749149135j,
+             0.9999999164704483-0.00033278721023225985j,
+             0.9999994455870466-0.0008573598787775811j,
+             0.9999963201854518-0.002208812789960272j,
+             0.9999755761485756-0.005690487871025065j,
+             0.9998379035432662-0.014659051083653684j,
+             0.9989246706803299-0.03774287852774699j,
+             0.9928871918753837-0.09684080299079524j,
+             0.9538456946366823-0.24287440441692343j,
+             0.7340692184563671-0.5282641958643781j])
+
+    def tearDown(self):
+        del self.frequencies
+        del self.known_response
+
+    def test_omc_dcpd_transimpedence_amplifier_response(self):
+        C = pydarm.sensing.SensingModel('''
+[metadata]
+[interferometer]
+[sensing]
+omc_path_names = A
+omc_meas_p_trans_amplifier = 13.7e3, 17.8e3
+''')
+        test_response = C.omc_dcpd_transimpedence_amplifier_response(
+            'A', self.frequencies)
+        for n in range(len(self.frequencies)):
+            self.assertAlmostEqual(
+                np.abs(test_response[n]),
+                np.abs(self.known_response[n]))
+            self.assertAlmostEqual(
+                np.angle(test_response[n], deg=True),
+                np.angle(self.known_response[n], deg=True))
+
+
+class TestOMCDCPDWhiteningResponse(unittest.TestCase):
+
+    def setUp(self):
+        self.frequencies = np.logspace(0, np.log10(5000.), 10)
+        self.known_response = np.array(
+            [0.9999999840941629-0.00014897328868210543j,
+             0.9999998944277334-0.00038380006810453216j,
+             0.9999992992823787-0.000988784271267042j,
+             0.999995349116734-0.002547399857003313j,
+             0.9999691309991117-0.006562759807444474j,
+             0.9997951317032564-0.016905766838847664j,
+             0.9986410763986874-0.0435222188297893j,
+             0.9910178637120639-0.11157937182260591j,
+             0.9419873353388445-0.2783804818601435j,
+             0.6742077533753816-0.5876961400786618j])
+
+    def tearDown(self):
+        del self.frequencies
+        del self.known_response
+
+    def test_omc_dcpd_whitening_response(self):
+        C = pydarm.sensing.SensingModel('''
+[metadata]
+[interferometer]
+[sensing]
+omc_path_names = A
+whitening_mode_names = test
+omc_meas_p_whitening_test = 11.346e3, 32.875e3, 32.875e3
+''')
+        test_response = C.omc_dcpd_whitening_response(
+            'A', self.frequencies)
+        for n in range(len(self.frequencies)):
+            self.assertAlmostEqual(
+                np.abs(test_response[n]),
+                np.abs(self.known_response[n]))
+            self.assertAlmostEqual(
+                np.angle(test_response[n], deg=True),
+                np.angle(self.known_response[n], deg=True))
+
+
 class TestOMCAnalaogDCPDReadoutResponse(unittest.TestCase):
 
     def setUp(self):
@@ -356,29 +435,29 @@ single_pole_approximation_delay_correction = -12e-6
                          deg=True))
 
 
-class TestKnownSensingTermsForMeasurement(unittest.TestCase):
+class TestSensingResidual(unittest.TestCase):
 
     def setUp(self):
         # Pre-computed values
         self.frequencies = np.logspace(0, np.log10(5000.), 10)
-        self.known_terms = np.array(
-            [1.0000003831992843-0.0007929858798234663j,
-             0.9999986255804101-0.002042969515282428j,
-             0.9999869597043105-0.005263286282389931j,
-             0.9999095302557973-0.013559488366496985j,
-             0.9993956411093267-0.034927799074183925j,
-             0.995986356934377-0.08988999137528692j,
-             0.9734269429695691-0.22996802633705746j,
+        self.known_residual = np.array(
+            [1.0000003831992847-0.0007929858798234665j,
+             0.9999986255804106-0.002042969515282433j,
+             0.9999869597043107-0.0052632862823899385j,
+             0.9999095302557972-0.013559488366496989j,
+             0.9993956411093269-0.03492779907418393j,
+             0.995986356934377-0.08988999137528689j,
+             0.9734269429695694-0.22996802633705743j,
              0.8267642616923517-0.5649608162253222j,
-             -0.0041816426639013065-1.003311656263212j,
-             -0.19691508479464048+0.779652195223653j])
+             -0.004181642663901519-1.003311656263212j,
+             -0.19691508479464048+0.7796521952236531j])
 
     def tearDown(self):
         del self.frequencies
-        del self.known_terms
+        del self.known_residual
 
-    def test_known_sensing_terms_for_measurement(self):
-        """ Test the computation of the known sensing parameters """
+    def test_sensing_residual(self):
+        """ Test the computation of the sensing function """
         C = pydarm.sensing.SensingModel('''
 [metadata]
 [interferometer]
@@ -403,14 +482,13 @@ omc_compensation_filter_bank = OMC_DCPD_A, OMC_DCPD_B
 omc_compensation_filter_modules_in_use = 4: 4
 omc_compensation_filter_gain = 1, 1
 ''')
-        correction_terms = C.known_sensing_terms_for_measurement(
-            self.frequencies)
+        C_res = C.sensing_residual(self.frequencies)
         for n in range(len(self.frequencies)):
             self.assertAlmostEqual(
-                np.abs(correction_terms[n]) / np.abs(self.known_terms[n]), 1.0)
+                np.abs(C_res[n]) / np.abs(self.known_residual[n]), 1.0)
             self.assertAlmostEqual(
-                np.angle(correction_terms[n], deg=True) -
-                np.angle(self.known_terms[n], deg=True), 0.0)
+                np.angle(C_res[n], deg=True) -
+                np.angle(self.known_residual[n], deg=True), 0.0)
 
 
 class TestComputeSensing(unittest.TestCase):
@@ -474,129 +552,5 @@ omc_compensation_filter_gain = 1, 1
                 np.angle(self.known_sensing[n], deg=True), 0.0)
 
 
-class TestSensingResidual(unittest.TestCase):
-
-    def setUp(self):
-        # Pre-computed values
-        self.frequencies = np.logspace(0, np.log10(5000.), 10)
-        self.known_residual = np.array(
-            [1.0000003831992847-0.0007929858798234665j,
-             0.9999986255804106-0.002042969515282433j,
-             0.9999869597043107-0.0052632862823899385j,
-             0.9999095302557972-0.013559488366496989j,
-             0.9993956411093269-0.03492779907418393j,
-             0.995986356934377-0.08988999137528689j,
-             0.9734269429695694-0.22996802633705743j,
-             0.8267642616923517-0.5649608162253222j,
-             -0.004181642663901519-1.003311656263212j,
-             -0.19691508479464048+0.7796521952236531j])
-
-    def tearDown(self):
-        del self.frequencies
-        del self.known_residual
-
-    def test_sensing_residual(self):
-        """ Test the computation of the sensing function """
-        C = pydarm.sensing.SensingModel('''
-[metadata]
-[interferometer]
-[sensing]
-x_arm_length = 3994.4704
-y_arm_length = 3994.4692
-coupled_cavity_optical_gain = 3.22e6
-coupled_cavity_pole_frequency = 410.6
-detuned_spring_frequency = 4.468
-detuned_spring_Q = 52.14
-sensing_sign = 1
-is_pro_spring = True
-anti_aliasing_rate_string = 16k
-anti_aliasing_method      = biquad
-analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
-omc_meas_p_trans_amplifier   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
-whitening_mode_names = test, test
-omc_meas_p_whitening_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
-super_high_frequency_poles_apparent_delay = 0, 0
-gain_ratio = 1, 1
-balance_matrix = 1, 1
-omc_path_names = A, B
-single_pole_approximation_delay_correction = -12e-6
-adc_gain = 1638.001638001638, 1638.001638001638
-omc_compensation_filter_file = test/H1OMC_1239468752.txt
-omc_compensation_filter_bank = OMC_DCPD_A, OMC_DCPD_B
-omc_compensation_filter_modules_in_use = 4: 4
-omc_compensation_filter_gain = 1, 1
-''')
-        C_res = C.sensing_residual(self.frequencies)
-        for n in range(len(self.frequencies)):
-            self.assertAlmostEqual(
-                np.abs(C_res[n]) / np.abs(self.known_residual[n]), 1.0)
-            self.assertAlmostEqual(
-                np.angle(C_res[n], deg=True) -
-                np.angle(self.known_residual[n], deg=True), 0.0)
-
-
-class TestDisplacementToCalcsResidual(unittest.TestCase):
-
-    def setUp(self):
-        # Pre-computed values
-        self.frequencies = np.logspace(0, np.log10(5000.), 10)
-        self.known_tf = np.array(
-            [3120586.3412569533+655915.5270972404j,
-             3197195.229554253+345256.84788082656j,
-             3218163.8630940774-81180.35305564835j,
-             3219467.296264217-64484.499226105225j,
-             3215576.1797485715-171881.23339198926j,
-             3188897.081164594-445606.88013708417j,
-             3015951.3533525704-1126845.0886107476j,
-             1959665.9094738844-2573950.8323788345j,
-             -2480555.5491107344-2268895.642652672j,
-             3361273.3553359923-1392363.5366565972j])
-
-    def tearDown(self):
-        del self.frequencies
-        del self.known_tf
-
-    def test_displacement_to_calcs(self):
-        """ Test the computation of the displacement to CALCS """
-        C = pydarm.sensing.SensingModel('''
-[metadata]
-[interferometer]
-[sensing]
-x_arm_length = 3994.4704
-y_arm_length = 3994.4692
-coupled_cavity_optical_gain = 3.22e6
-coupled_cavity_pole_frequency = 410.6
-detuned_spring_frequency = 4.468
-detuned_spring_Q = 52.14
-sensing_sign = 1
-is_pro_spring = True
-anti_aliasing_rate_string = 16k
-anti_aliasing_method      = biquad
-analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
-omc_meas_p_trans_amplifier   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
-whitening_mode_names = test, test
-omc_meas_p_whitening_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
-super_high_frequency_poles_apparent_delay = 0, 0
-gain_ratio = 1, 1
-balance_matrix = 1, 1
-omc_path_names = A, B
-single_pole_approximation_delay_correction = -12e-6
-adc_gain = 1638.001638001638, 1638.001638001638
-omc_compensation_filter_file = test/H1OMC_1239468752.txt
-omc_compensation_filter_bank = OMC_DCPD_A, OMC_DCPD_B
-omc_compensation_filter_modules_in_use = 4: 4
-omc_compensation_filter_gain = 1, 1
-gds_c_foton_invsensing_tf = \
-  test/2019-04-04_H1CALCS_InverseSensingFunction_Foton_SRCD-2N_Gain_tf.txt
-''')
-        disp_to_calcs_tf = C.displacement_to_calcs_residual(self.frequencies)
-        for n in range(len(self.frequencies)):
-            self.assertAlmostEqual(
-                np.abs(disp_to_calcs_tf[n]) / np.abs(self.known_tf[n]), 1.0)
-            self.assertAlmostEqual(
-                np.angle(disp_to_calcs_tf[n], deg=True) -
-                np.angle(self.known_tf[n], deg=True), 0.0)
-
-
 if __name__ == '__main__':
     unittest.main()
